<?php

namespace Modules\WhatsappSupport\Http\Controllers;

use Brian2694\Toastr\Facades\Toastr;
use Carbon\Carbon;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\WhatsappSupport\Entities\Agents;
use Modules\WhatsappSupport\Entities\Settings;
use Modules\WhatsappSupport\Http\Requests\AgentCreation;
use Modules\WhatsappSupport\Traits\ImageStore;

class AgentController extends Controller
{
    public function index()
    {
        $agents = Agents::all();
        return view('whatsappsupport::agents.index', compact('agents'));
    }

    public function create()
    {
        return view('whatsappsupport::agents.create');
    }

    public function store(AgentCreation $request)
    {
        if($request->always_available == '0'){
            if (!$request->has('day')){
                session()->flash('error', 'Please select/check at least one working day!');
                return redirect()->back()->withInput();
            }
        }

        $imagepath = 'public/whatsapp-support/demo-avatar.jpg';

        if($request->hasFile('avatar')){
            $imagepath = ImageStore::saveAvatarImage($request->avatar);
        }
        $agent = Agents::create([
            'name' => $request->name,
            'avatar' => $imagepath,
            'designation' => $request->designation,
            'number' => $request->number,
            'always_available' => $request->always_available,
            'status' => $request->always_available,
        ]);

        if($request->always_available == '0') {
            foreach ($request->day as $day){
                $index = $this->getDayIndex($day);
                $agent->times()->updateOrCreate(
                    [
                        'agent_id' => $agent->id,
                        'day' => $day
                    ], [
                    'day' => $day,
                    'start' => Carbon::parse($request->start[$index])->format('H:i:s'),
                    'end' => Carbon::parse($request->end[$index])->format('H:i:s')
                ]);
            }
        }

        Toastr::success('Agent Added', 'Success');

        return redirect()->back();

    }

    public function show($id)
    {
        $agent = Agents::find($id);
        return view('whatsappsupport::agents.show', compact('agent'));
    }

    public function edit($id)
    {
        return view('whatsappsupport::edit');
    }

    public function update(AgentCreation $request)
    {
        $agent = Agents::find($request->agent_id);

        if($request->always_available == '0'){
            if (!$request->has('day')){
                session()->flash('error', 'Please select/check at least one working day!');
                return redirect()->back();
            }
        }

        $imagepath = $agent->avatar;
        if($request->hasFile('avatar')){
            $imagepath = ImageStore::saveAvatarImage($request->avatar);
        }
        $agent->update([
            'name' => $request->name,
            'avatar' => $imagepath,
            'designation' => $request->designation,
            'number' => $request->number,
            'always_available' => $request->always_available,
            'status' => $request->status,
        ]);

        if($request->always_available == '0') {
            $agent->times()->delete();
            foreach ($request->day as $day){
                $index = $this->getDayIndex($day);
                $agent->times()->updateOrCreate(
                    [
                        'agent_id' => $agent->id,
                        'day' => $day
                    ], [
                    'day' => $day,
                    'start' => Carbon::parse($request->start[$index])->format('H:i:s'),
                    'end' => Carbon::parse($request->end[$index])->format('H:i:s')
                ]);
            }
        }

        Toastr::success('Agent Updated', 'Success');

        return redirect()->back();
    }

    public function destroy($id)
    {
        $agent = Agents::find($id);
        $agent->times()->delete();
        $agent->delete();
        Toastr::success('Agent Deleted!', 'Success');
        return redirect()->back();
    }

    public function getDayIndex($day)
    {
        switch ($day){
            case 'Sunday' :
                return 0;
            case 'Monday' :
                return 1;
            case 'Tuesday' :
                return 2;
            case 'Wednesday' :
                return 3;
            case 'Thursday' :
                return 4;
            case 'Friday' :
                return 5;
            case 'Saturday' :
                return 6;
        }
    }
}
